#include <string.h>
#include <QtGui/QImage>
#include <QtGui/qcolor.h>
#include <assert.h>
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <iostream>
using namespace std;
float correlate(float* X, float* Y, int n)
{
  // first translate the mean
  assert(X);
  assert(Y);
  float* x = new float[n];
  float* y = new float[n];
  assert(x);
  assert(y);
  // calculate the averages
  float x_avg=0,y_avg=0;
  for(int i = 0 ; i < n ; i ++)
    {
      x_avg+=X[i];
      y_avg+=Y[i];
    }
  x_avg/=n;
  y_avg/=n;
  for(int i = 0 ; i < n ; i ++)
    {
      x[i]=X[i]-x_avg;
      y[i]=Y[i]-y_avg;
    }
  // the average of the dataset is now 0
  x_avg=y_avg=0;
  double x_var=0,y_var=0;
  for(int i = 0 ; i < n; i++)
    {
      double x_delta=x[i]-x_avg;
      double y_delta=y[i]-y_avg;
      x_avg+=x_delta/(i+1);
      y_avg+=y_delta/(i+1);
      x_var+=x_delta*(x[i]-x_avg);
      y_var+=y_delta*(y[i]-y_avg);
    }
  // printf("%g x %g\n",x_avg,y_avg);
  float x_std=sqrt(x_var);
  float y_std=sqrt(y_var);
  // the covariance
  float r=0;
  for(int i = 0 ; i < n; i++)
    r+=x[i]*y[i];
  r/=x_std*y_std;
  // return the correlation value
  delete x,y;
  return r;
}

// a routine to read an image into memory
// the returned value is a 2 dimensional array 
// of size sx, sy
unsigned char** read_image(const char* filename, int& sx, int&sy)
{
  QImage img(filename);
  if (img.isNull()) return NULL;
  cerr << "Reading image " << filename << "\n"; 
  if (sx==0)
    {
      sx=img.width();
      sy=img.height();
    }
  else
    {
      assert(sx==img.width());
      assert(sy==img.height());
    }
  unsigned char** result=new unsigned char*[sx];
  for(int i = 0 ; i < sx; i++)
    {
      result[i]=new unsigned char[sy];
      assert(result[i]);
    }
  assert(result);
  // convert everything to grey values
  for(int x = 0 ; x < sx; x++)
    for(int y = 0; y < sy; y++)
      {
	QRgb rgb=img.pixel(x,y);
	unsigned char intensity=qGray(rgb);
	result[x][y]=intensity;
      }
  return result;
}

string** read_csv(const char* filename, int& sx, int&sy)
{
  // we scan the file twice because we are lazy.
  FILE * f = fopen(filename, "rb");
  assert(f);
  char field[5000];
  int x = 0;
  int y = 0;
  cerr << "Reading file " << filename;
  int colcount=0;
  int columns=0;
  unsigned char sep='\t';
  while(true)
    {
      int c = fgetc(f);
      // we end the current field at EOF, newline or tabs
      if (c==-1 || c==sep || c=='\n')
	{
	  field[x++]=0;
	  columns++;
	  // row->push_back(field);
	  x=0;
	}
      // we end the line at EOF or newline
      if (c==-1 || c=='\n')
	{
	  if (columns>colcount) colcount=columns;
	  columns=0;
	  // addRow(row);
	  // row = new Row();
	  y++;
	}
      // we end the process at EOF
      if (c==-1)
	break;
      if (c!=sep && c!='\n' && c!=-1)
	{
	  //field[x++]=c;
	  //assert(x<5000);
	}
    }
  
  string** result=new string*[colcount];
  for(int i = 0 ; i < colcount; i++)
    result[i]=new string[y];
  
  columns=0;
  y=0;
  fseek(f,0,SEEK_SET);
  while(true)
    {
      int c = fgetc(f);
      // we end the current field at EOF, newline or tabs
      if (c==-1 || c==sep || c=='\n')
	{
	  field[x++]=0;
	  result[columns++][y]=field;
	  x=0;
	}
      // we end the line at EOF or newline
      if (c==-1 || c=='\n')
	{
	  columns=0;
	  y++;
	}
      // we end the process at EOF
      if (c==-1)
	break;
      if (c!=sep && c!='\n' && c!=-1)
	{
	  field[x++]=c;
	  assert(x<5000);
	}
    }
  cerr<<": "<<colcount<<" columns, " << y << " lines [done]\n";
  fclose(f);
  sx=colcount;
  sy=y;
  return result;
}

int main(int argc, char* argv[])
{
  // read the csv file
  int csv_sx,csv_sy;
  string **csv=read_csv(argv[1],csv_sx,csv_sy);

  unsigned char** image;
  int sx=0,sy=0;
  image[i]=read_image(csv[0][1].c_str(),sx,sy);
  printf("Image size is %dx%d\n",sx,sy);
  
  char tmp[200];
  sprintf(tmp,"param-%d.jpg",p);
  O.save(tmp);
}
